pro make_global_current_kml,modis_name
;---------------------------------------------------------------------
; Program to make a single .kml file that points to the top level
; super overlay images/kml for each of the tiles in the CURRENT global
; MODIS imagery.  The global tiles are numbered from 36 through 611 
; (576 in total).  This program must be executed each time any tile(s)
; is updated to check for new links, missing data, etc.
;
; INPUT:
;  modis_name: This is the name of the new MODIS image, only needed to
;   determine the satellite (Terra/Aqua) being considered.
;
; OUTPUT:
;  A kml file that will link to all of the top level super overlay
;  files and imagery.  Naming convention will be current.satellite,
;  e.g., current.aqua.kml or current.terra.kml.
;
; HARDWIRED:
;  Path to the current superoverlay tiles (e.g., '../kml/current/')
;
; AUTHOR: Amato Evan - CIMSS/UW-Madison amatoe@ssec.wisc.edu
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; Define the path to the super overlay folders for this day
;---------------------------------------------------------------------
satellite_id = strmid(modis_name,8,4) ;terr or aqua
if(satellite_id eq 'terr')then satellite_id = 'terra'
kml_name = 'current.'+satellite_id+'.kml'
kml_control_path = '../kml/current/'

;---------------------------------------------------------------------
; Determine which of the global tiles have imagery to be linked to
;---------------------------------------------------------------------
global_tiles = file_search(kml_control_path,'*'+satellite_id)
n_tiles = n_elements(global_tiles)
id_tiles = intarr(n_tiles)
for i = 0, n_tiles-1 do id_tiles[i] = 1*strmid(global_tiles[i],19,3)

;---------------------------------------------------------------------
; Define the MODIS tiles lat/lon limits
;---------------------------------------------------------------------
region_coords = dblarr(n_tiles,4)
gridindex_to_latlon,id_tiles,10,lat,lon
region_coords[*,0] = lat
region_coords[*,1] = lon
region_coords[*,2] = temporary(lat)+10.
region_coords[*,3] = temporary(lon)+10.

;---------------------------------------------------------------------
; Make the kml file that will point to all the top level super overlay
; images.
;---------------------------------------------------------------------
openw,u,kml_control_path+kml_name,/get_lun

;write the opening lines
printf,u,'<?xml version="1.0" encoding="UTF-8"?>'
printf,u,'<kml xmlns="http://earth.google.com/kml/2.2">'
printf,u,'  <Document>'
printf,u,'    <Region>'
printf,u,'      <Lod>'
printf,u,'        <minLodPixels>128</minLodPixels><maxLodPixels>-1</maxLodPixels>'
printf,u,'      </Lod>'
printf,u,'      <LatLonAltBox>'
printf,u,'        <north>'+strcompress(max(region_coords[*,2]))+'</north>'
printf,u,'        <south>'+strcompress(min(region_coords[*,0]))+'</south>'
printf,u,'        <east>'+strcompress(max(region_coords[*,3]))+'</east>'
printf,u,'        <west>'+strcompress(min(region_coords[*,1]))+'</west>'
printf,u,'      </LatLonAltBox>'
printf,u,'    </Region>'

;---------------------------------------------------------------------
; loop through and link to sub-directories
;---------------------------------------------------------------------
for i = 0, n_tiles-1 do begin

  ;-------------------------------------------------------------------
  ; Determine the "uppermost" kml super overlay tile by doing a search
  ;-------------------------------------------------------------------
  tile_name = strcompress(id_tiles[i],/remove_all)
  if(id_tiles[i] lt 10)then tile_name = '00'+ $
                                  strcompress(id_tiles[i],/remove_all)
  if(id_tiles[i] ge 10 and id_tiles[i] lt 100)then $
                  tile_name = '0'+strcompress(id_tiles[i],/remove_all)
  full_path = kml_control_path+'tile'+tile_name+'_'+satellite_id+'/'

  tmp = file_search(full_path,'*.kml')
  tiles = intarr(n_elements(tmp))
  for s = 1, n_elements(tmp)-1 do begin
    tnum = strmid(tmp[s],strlen(tmp[s])-7,3)
    if(strmid(tnum,0,1) eq '/' or strmid(tnum,0,1) eq 'a')then $
                                              tnum = strmid(tnum,1,2)
    if(strmid(tnum,0,1) eq '/')then tnum = strmid(tnum,1,2)
    tiles[s] = tnum*1
  endfor

  upper_tile = strcompress(max(tiles),/remove_all)
  if(max(tiles) lt 10)then upper_tile = '00'+ $
                                   strcompress(max(tiles),/remove_all)
  if(max(tiles) ge 10 and max(tiles) lt 100)then $ 
                  upper_tile = '0'+strcompress(max(tiles),/remove_all)
  ;-------------------------------------------------------------------

  printf,u,'    <NetworkLink>'
  printf,u,'      <name>'+upper_tile+'</name>'
  printf,u,'      <Region>'
  printf,u,'        <Lod>'
  printf,u,'          <minLodPixels>128</minLodPixels><maxLodPixels>-1</maxLodPixels>'
  printf,u,'        </Lod>'
  printf,u,'        <LatLonAltBox>'
  printf,u,'          <north>'+strcompress(region_coords[i,2])+'</north>'
  printf,u,'          <south>'+strcompress(region_coords[i,0])+'</south>'
  printf,u,'          <east>'+strcompress(region_coords[i,3])+'</east>'
  printf,u,'          <west>'+strcompress(region_coords[i,1])+'</west>'
  printf,u,'        </LatLonAltBox>'
  printf,u,'      </Region>'
  printf,u,'      <Link>'

  printf,u,'        <href>tile'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.kml</href>'

  printf,u,'        <viewRefreshMode>onRegion</viewRefreshMode>'
  printf,u,'      </Link>'
  printf,u,'    </NetworkLink>'
  
  printf,u,'        <GroundOverlay>'
  printf,u,'          <drawOrder>0</drawOrder>'
  printf,u,'          <Icon>'

  printf,u,'            <href>tile'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.jpg</href>'

  printf,u,'          </Icon>'
  printf,u,'          <LatLonBox>'
  printf,u,'            <north>'+strcompress(region_coords[i,2])+'</north>'
  printf,u,'            <south>'+strcompress(region_coords[i,0])+'</south>'
  printf,u,'            <east>'+strcompress(region_coords[i,3])+'</east>'
  printf,u,'            <west>'+strcompress(region_coords[i,1])+'</west>'
  printf,u,'          </LatLonBox>'
  printf,u,'        </GroundOverlay>'
endfor

;end & close the file
printf,u,'      </Document>'
printf,u,'    </kml>'

close,u
free_lun,u

end
;-------------------------------------------------------------
